#include "common_defs.h"

#define REPLY_LENG                 5 //     
#define CONTROLLER_ADDRESS      0xE0 //  
#define START_BYTE              0xFE //  
#define STOP_BYTE               0xFD //  
#define CMD_READ_FREQ           0x03 //  -   
#define CMD_READ_MODE           0x04 //  -   
#define CMD_READ_SMETER1        0x15 //  -  S-
#define CMD_READ_SMETER2        0x02 //  -  S- ()
#define SHOW_DELAY              1000 //       
#define EEPROM_START_ADDRESS    0x70 // ,      
#define EE_CS1                  EEPROM_START_ADDRESS   //    
#define EE_CS2                  EEPROM_START_ADDRESS+1 //    
#define EE_NUM_OF_RADIOS        EEPROM_START_ADDRESS+2 // - ""  
#define EE_INFO_START           EEPROM_START_ADDRESS+3 //      ""      
#define START_RADIO_ADDRESS     0x01                   // ,      ,     01H...DFH
#define STOP_RADIO_ADDRESS      CONTROLLER_ADDRESS     // ,      
#define NUMBER_OF_STORED_RADIOS 10                     // -    ""

#define LED_FIND_CONNECT        PB0 //   ()/ ( )
#define LED_144MHZ              PD5 //    144
#define LED_430MHZ              PD6 //    430
#define LED_1200MHZ             PD7 //    1200

extern const UINT8 PROGMEM hex_mass[];
const UINT16 dec_mass[]           = {1000, 100, 10, 1};
const UINT32 dec_mass10[]         = {1000000000, 100000000, 10000000, 1000000, 100000, 10000, 1000, 100, 10, 1};

UINT8   radio_address;             //   00H...F9H,  ,  ,   E0H -  
UINT16  baud_rate;                 //   UART
UINT32  err_delay;                 //         UART
UINT8   err_flag;                  //      UART
UINT8   read_buffer[12];           //      UART
UINT16  frequency;                 //  

const UINT16 baud[]               = {19200, 9600, 1200, 4800, 300};
UINT8 read_freq[]                 = {START_BYTE, START_BYTE, 0, CONTROLLER_ADDRESS, CMD_READ_FREQ, STOP_BYTE}; //   
UINT8 read_freq_rply_ok[]         = {START_BYTE, START_BYTE, CMD_READ_FREQ, STOP_BYTE}; //    "  "
UINT8 read_freq_rply_ok2[]        = {START_BYTE, START_BYTE, CONTROLLER_ADDRESS, CMD_READ_FREQ, STOP_BYTE}; //    "  " ( 2)
UINT8 read_freq_rply[]            = {START_BYTE, START_BYTE, CONTROLLER_ADDRESS, 0, CMD_READ_FREQ};   //      "  "
UINT8 read_mod[]                  = {START_BYTE, START_BYTE, 0, CONTROLLER_ADDRESS, CMD_READ_MODE, STOP_BYTE}; //    
UINT8 read_mod_rply_ok[]          = {START_BYTE, START_BYTE, CMD_READ_MODE, STOP_BYTE};
UINT8 read_mod_rply_ok2[]         = {START_BYTE, START_BYTE, CONTROLLER_ADDRESS, CMD_READ_MODE, STOP_BYTE};
UINT8 read_mod_rply[]             = {START_BYTE, START_BYTE, CONTROLLER_ADDRESS, 0, CMD_READ_MODE};
UINT8 read_smeter[]               = {START_BYTE, START_BYTE, 0, CONTROLLER_ADDRESS, CMD_READ_SMETER1, CMD_READ_SMETER2, STOP_BYTE}; //   S-
UINT8 read_smeter_rply_ok[]       = {START_BYTE, START_BYTE, CMD_READ_SMETER2, STOP_BYTE}; //    " S-"
UINT8 read_smeter_rply_ok2[]      = {START_BYTE, START_BYTE, CMD_READ_SMETER1, CMD_READ_SMETER2, STOP_BYTE}; //    " S-" ( 2)
UINT8 read_smeter_rply[]          = {START_BYTE, START_BYTE, CONTROLLER_ADDRESS, 0, CMD_READ_SMETER1, CMD_READ_SMETER2};

const MODE mode[] =
{
   {0x0001, "LSB \0"}, {0x0002, "LSBN\0"}, {0x0003, "LSBN\0"},
   {0x0101, "USB \0"}, {0x0102, "USBN\0"}, {0x0103, "USBN\0"},
   {0x0201, "AM  \0"}, {0x0202, "AMN \0"}, {0x0203, "AMNN\0"},
   {0x0301, "CW  \0"}, {0x0302, "CWN \0"}, {0x0303, "CWNN\0"},
   {0x0401, "RTTY\0"}, {0x0402, "RTTN\0"}, {0x0403, "RTTN\0"},
   {0x0501, "FM  \0"}, {0x0502, "FMN \0"}, {0x0503, "FMNN\0"},
   {0x0601, "WFM \0"}, {0x0602, "WFM \0"}, {0x0603, "WFM \0"},
   {0x0701, "CWR \0"}, {0x0702, "CWRN\0"}, {0x0703, "CWRN\0"},
   {0x0801, "RTTR\0"}, {0x0802, "RTRN\0"}, {0x0803, "RTRN\0"},
   {0x1701, "DV  \0"}, {0x1702, "DVN \0"}, {0x1703, "DVNN\0"}
};

char  *search[]   = {"   \0", ".  \0", ".. \0", "...\0"}; //      " ..."  "..."

#if (1 == WINSTAR)
    #define SMETER_START_CODE 0x03
    char  search2[]   = {'\x01', '/', '-', '\x02'};       /*   |/-\   WINSTAR */
#else
    #define SMETER_START_CODE 0x94
    char  search2[]   = {'|', '/', '-', '\\'};            /*   |/-\    */
#endif

/*****************************************
 *                 *
 *****************************************/
void CLR_ALL_LEDS(void)
{
   CLR_LED_144MHZ();
   CLR_LED_430MHZ();
   CLR_LED_1200MHZ();
}

/*****************************************
 *      LED_1200MHZ   *
 *****************************************/
void inline SET_LED_1200MHZ(void)
{
   PORTD |= _BV(LED_1200MHZ);
}

/*****************************************
 *      LED_1200MHZ   *
 *****************************************/
void inline CLR_LED_1200MHZ(void)
{
   PORTD &= ~_BV(LED_1200MHZ);
}

/*****************************************
 *      LED_430MHZ    *
 *****************************************/
void inline SET_LED_430MHZ(void)
{
   PORTD |= _BV(LED_430MHZ);
}

/*****************************************
 *      LED_430MHZ    *
 *****************************************/
void inline CLR_LED_430MHZ(void)
{
   PORTD &= ~_BV(LED_430MHZ);
}

/*****************************************
 *      LED_144MHZ    *
 *****************************************/
void inline SET_LED_144MHZ(void)
{
   PORTD |= _BV(LED_144MHZ);
}

/*****************************************
 *      LED_144MHZ    *
 *****************************************/
void inline CLR_LED_144MHZ(void)
{
   PORTD &= ~_BV(LED_144MHZ);
}

/*****************************************
 *   LED_FIND_CONNECT *
 *****************************************/
void inline SET_LEDFindConnect(void)
{
   PORTB |= _BV(LED_FIND_CONNECT);
}

/*****************************************
 *   LED_FIND_CONNECT *
 *****************************************/
void inline CLR_LEDFindConnect(void)
{
   PORTB &= ~_BV(LED_FIND_CONNECT);
}

/********************************************
 *   LED_FIND_CONNECT *
 ********************************************/
void TOGGLE_LEDFindConnect(void)
{
   if (PINB & _BV(LED_FIND_CONNECT))
      CLR_LEDFindConnect();
   else
      SET_LEDFindConnect();
}

#if (1 == _DEBUG)
/*****************************************************
 *      16- HEX        *
 *****************************************************/
void hex_print_16(INT16 value)
{
   lcd_print_hex_byte((UINT8)((value >> 8) & 0x00FF));
   lcd_print_hex_byte((UINT8)(value & 0x00FF));
}

/*****************************************************
 *            *
 *****************************************************/
void print_reply()
{
   UINT8 i;
   UINT8 byte;
   UINT8 count;

   for (i=0; i < sizeof(read_freq); i++)
   {
      UARTTransmit(read_freq[i]);
   }

   Locate(1, 0);
   i = read_line();
   for (count = 0; count < i; count++)
   {
      byte = read_buffer[count];
      lcd_print_hex_byte(byte);
   }
   Locate(2, 1);
   i = read_line();
   for (count = 0; count < i; count++)
   {
      byte = read_buffer[count];
      lcd_print_hex_byte(byte);
   }
}
#endif

/******************************************************************
 *                                           *
 ******************************************************************/
void IOinit(void)
{
   ACSR   = _BV(ACD);                            //     
   DDRC   = 0xFF;                                //   C  
   PORTC  = 0x00;                                //    C   0
   DDRD   = _BV(PD1)|_BV(LED_144MHZ)| \
            _BV(LED_430MHZ)|_BV(LED_1200MHZ);    // PD1 (TXD)  ,  - 
   PORTD |= _BV(PD0) | _BV(PD1);                 //    PD0  PD1
   DDRB   = _BV(LED_FIND_CONNECT);               //  LED_FIND_CONNECT
}

/*****************************
 *     UART     *
 *****************************/
void UARTinit(UINT32 Baud)
{
   cli();
   UINT16 BaudRate = F_CPU / (16 * Baud) - 1;
   UBRRH = (UINT8) (BaudRate>>8);
   UBRRL = (UINT8) BaudRate;
   /* Enable receiver and transmitter */
   UCSRB = (1<<RXEN)|(1<<TXEN);
   /* Set frame format: 8 data, 1 stop bit */
   UCSRC = (1<<URSEL)|(3<<UCSZ0);

   //       baud_rate
   switch(Baud)
   {
      case 300:
         err_delay = 120000;
         break;
      case 1200:
         err_delay = 60000;
         break;
      case 4800:
         err_delay = 50000;
         break;
      case 9600:
         err_delay = 40000;
         break;
      case 19200:
         err_delay = 30000;
         break;
      default:
         err_delay = 120000;
         break;
   }
   sei();
}

/*****************************
 *       UART   *
 *****************************/
UINT8 UARTReceive(void)
{
   UINT32 ed = err_delay;

   for(;;)
   {
      if (UCSRA & 0b10000000)
      {
         break;           //  
      }
      if (0 == --ed)
      {
         err_flag |= 1;   //  -  !
         break;
      }
   }
   return UDR;
}

/*****************************
 *      UART   *
 *****************************/
void UARTTransmit(UINT8 data)
{
   while (!(UCSRA & 0b00100000));
   UDR = data;
}

/*****************************
 *      UART   *
 *****************************/
UINT8 read_line(void)
{
   UINT8 byte;
   UINT8 counter = 0;
   err_flag = 0;

   for (;;)
   {
      byte = UARTReceive();
      read_buffer[counter++] = byte;
      if ((STOP_BYTE == byte) ||
         (counter >= sizeof(read_buffer))) break;
   }
   return counter;
}

/**********************************************
 *      0xFE        *
 *    (  ) *
 *    0       *
 **********************************************/
UINT8 STOPsearch(UINT8 start_index)
{
   for (;;)
   {
      if (++start_index >= sizeof(read_buffer)) return 0;
      if (STOP_BYTE == read_buffer[start_index]) return start_index;
   }
}

/**********************************
 *       *
 **********************************/
void FreqPrint(void)
{
   UINT8 start_index;
   UINT8 stop_index;
   UINT8 index = 0;
   UINT8 high_nibble;
   UINT8 low_nibble;
   UINT8 print = 0;

   Locate(1, 4);
   start_index = REPLY_LENG; //   BCD      
   stop_index  = STOPsearch(start_index);

   frequency = BCD2Int32(&read_buffer[start_index])/1000000; //   ,   1000000
                                                             // (      UINT16)
                                                             //  1295000000   1295    ..

   if(0 == stop_index) return; // !!! -   !

   while (--stop_index >= start_index)
   {
      //   
      high_nibble = (read_buffer[stop_index] >> 4);
      if (0 != high_nibble)
      {
         print = 1;
         FreqPrintChar(high_nibble, &index);
      }
      else if (print)
      {
         FreqPrintChar(high_nibble, &index);
      }
      else
      {
         FreqPrintChar(0xFF, &index);
      }

      //   
      low_nibble  = (read_buffer[stop_index] & 0x0F);
      if (0 != low_nibble)
      {
         print = 1;
         FreqPrintChar(low_nibble, &index);
      }
      else if (print)
      {
         FreqPrintChar(low_nibble, &index);
      }
      else
      {
         FreqPrintChar(0xFF, &index);
      }
   }
}

/******************************************
 *          *
 ******************************************/
void FreqPrintChar(UINT8 byte, UINT8 * index)
{
   (*index)++;
   if (0xFF != byte)
   {
      lcd_print_char((UINT8)(__LPM(hex_mass + byte)));   //   
   }
   else
   {
      lcd_print_char(' ');   //   
   }
   if ((4 == *index) || (7 == *index)) lcd_print_char('.');
}

/*****************************************
 *        *
 *****************************************/
void ModePrint(void)
{
   UINT8  start_index;
   UINT8  stop_index;
   UINT8  i;
   UINT8  flag = 0;
   UINT16 md;

   start_index = REPLY_LENG; //   BCD      
   stop_index  = STOPsearch(start_index);

   if ((0 == stop_index)||(1 == stop_index)) return; // !!! -   !

   md = (UINT16)((UINT16)read_buffer[--stop_index]);
   md ^= (UINT16)(((UINT16)read_buffer[--stop_index]) << 8);

   Locate(1, 0);

   for (i = 0; i < (sizeof(mode)/sizeof(MODE)); i++) 
   {
      if (md == mode[i].mode_data)
      {
         lcd_print_str(mode[i].mode_str);
         flag = 1;
         break;
      }
   }
   if (0 == flag) lcd_print_str("... ");  //       ,    
}

/*****************************************
 *       S-       *
 *****************************************/
void SMeterPrint(void)
{
   UINT8  start_index;
   UINT8  stop_index;
   UINT16 smeter;
   UINT8  tmp;
   UINT8  i;
   UINT8  index = 0;
   UINT8  s_meter_buf[17];

   //     "3 "
   memset(&s_meter_buf[0], SMETER_START_CODE, sizeof(s_meter_buf));
   s_meter_buf[16] = 0; //   

   start_index = REPLY_LENG; //   BCD      
   stop_index  = STOPsearch(start_index);

   if ((0 == stop_index)||(1 == stop_index)) return; // !!! -   !

   smeter = (UINT16)((UINT16)read_buffer[--stop_index]);
   smeter ^= (UINT16)(((UINT16)read_buffer[--stop_index])<< 8);

   Locate(2, 0);
   BCD2cif(&smeter);                  // smeter -    S- (   00...FF)
   tmp = (UINT8)smeter >> 2;          //   4, ..   0...255   0...63

   for (i=0; i<(tmp>>2); i++)         //    4, ..   0...63   0...15
   {
      s_meter_buf[index++] = SMETER_START_CODE+3;    //    ||| (  )  
   }

   if (index > 0) s_meter_buf[index] = SMETER_START_CODE+(tmp&0x03); //    -   -  0x94 ... 0x97 - (..., |.., ||., |||)

   lcd_print_str((char*)&s_meter_buf[0]);
}

/*****************************
 *  BCD -> 16-    *
 *    BCD    *
 *****************************/
void BCD2cif(UINT16* smeter)
{
   UINT16 accum = 0;
   UINT16 value = *smeter;
   UINT8  tmp;
   UINT8  i;
   UINT8  buf[4];

   //  BCD  , . .      HEX-  .
   //       BCD,    ,    .
   tmp = (UINT8)((value >> 8) & 0x00FF);
   buf[0] = (UINT8)(__LPM(hex_mass + (tmp >> 4)));   //   
   buf[1] = (UINT8)(__LPM(hex_mass + (tmp & 0x0F))); //   
   tmp = (UINT8)(value & 0x00FF);
   buf[2] = (UINT8)(__LPM(hex_mass + (tmp >> 4)));   //   
   buf[3] = (UINT8)(__LPM(hex_mass + (tmp & 0x0F))); //   

   // ""   .     16- 
   for (i=0; i < 4; i++)
   {
      accum += (buf[i]-0x30) * dec_mass[i];
   }
   *smeter = accum;
}

/*************************************
 * BCD -> 32-   Int   *
 *   BCD   Icom *
 *                                   *
 *     10- BCD  *
 *         *
 * 9,999,999,999 - 9 .     *
 *   32-  -       *
 * 4.2      !    *
 *                                   *
 * p_mass -  5-      *
 * BCD-                         *
 *************************************/
UINT32 BCD2Int32(UINT8* p_mass)
{
   UINT32 accum = 0;
   UINT8  tmp;
   UINT8  i, counter = 0;
   UINT8  buf[10]; //  BCD    

   //  BCD  , . .      HEX- 
   //  (Icom Inc., !).
   //       BCD,     ,
   //    .
   for (i = 0; i < 5; i++)
   {
      tmp = p_mass[4-i];
      buf[counter++] = (UINT8)(__LPM(hex_mass+(tmp >> 4)));   //   
      buf[counter++] = (UINT8)(__LPM(hex_mass+(tmp & 0x0F))); //   
   }

   // ""   .     32- 
   for (i = 0; i < 10; i++)
   {
      accum += (buf[i]-0x30) * dec_mass10[i];
   }

   return accum;
}

/*****************************************
 *      *
 *****************************************/
void RAinit(UINT8 radio_address)
{
   read_freq[2]        = radio_address;
   read_freq_rply[3]   = radio_address;
   read_mod[2]         = radio_address;
   read_mod_rply[3]    = radio_address;
   read_smeter[2]      = radio_address;
   read_smeter_rply[3] = radio_address;
}

/******************************************
 *       *
 ******************************************/
void pause(void)
{
   UINT16 delay;
   switch(baud_rate)
   {
      case  300:
         delay = 1;
         break;
      case 1200:
         delay = 1;
         break;
      case  4800:
         delay = 10;
         break;
      case  9600:
         delay = 150;
         break;
      case 19200:
         delay = 200;
         break;
      default:
         delay = 200;
   }
   Wait_mS(delay);
}

/****************************************
 *      *
 ****************************************/
bool radio_search(UINT16 br)
{
   UINT8 address;
   UINT8 counter = 0;
   bool  result = false;

   cls();
   Locate(1, 0);
   lcd_print_str(": 01H   :");
   lcd_print_dec(br);
   Locate(2, 0);
   lcd_print_str(" ");
                                  
   UARTinit(br);

   for (address = START_RADIO_ADDRESS; address < STOP_RADIO_ADDRESS; address++)
   {
      TOGGLE_LEDFindConnect();
      Locate(0, 3);
      lcd_print_hex_byte(address);
      if (radio_check_exist(address))
      {
         Locate(2, 0);
         lcd_print_str(" !  ");
         Wait_mS(SHOW_DELAY);
         result = true;
         break;
      }
      counter++;
      Locate(2, 9);
      lcd_print_str(search[counter & 0x03]);
      Locate(2, 15);
      lcd_print_char(search2[counter & 0x03]);
   }

   radio_address = address;
   baud_rate     = br;

   return result;
}

/******************************************
 *      *
 *                        *
 ******************************************/
bool radio_check_connection(UINT8 ra)
{
   UINT8 j;
   bool  result = false;

   //  radio_address
   RAinit(ra);

   //   "" 3 
   for (j = 0; j < 3; j++)
   {
      if (radio_check_exist(ra))
      {
         result = true;
         break;
      }
      Wait_mS(10);
   }
   return result;
}

/******************************************
 *      *
 *                        *
 ******************************************/
bool radio_check_exist(UINT8 ra)
{
   UINT8 i;
   bool  result = false;

   //  radio_address
   RAinit(ra);

   //   
   for (i=0; i < sizeof(read_freq); i++)
   {
      UARTTransmit(read_freq[i]);
   }
   i = read_line();
   if ((0 == err_flag) && (i != 0) && (i != sizeof(read_buffer)))
   {
      if ((!memcmp(&read_buffer[0], &read_freq_rply_ok[0], sizeof(read_freq_rply_ok))) ||
          (!memcmp(&read_buffer[0], &read_freq_rply_ok2[0], sizeof(read_freq_rply_ok2)))) //      ...
      {
         i = read_line();
         if ((0 == err_flag) && (i != 0) && (i != sizeof(read_buffer)))
         {
            if (!memcmp(&read_buffer[0], &read_freq_rply[0], sizeof(read_freq_rply))) //       ...
            {
               result = true;
            }
         }
      }
   }
   return result;
}

/**********************************************************************
 *     ,      *
 **********************************************************************/
void get_data(UINT8* p_cmd, UINT8* p_rply_ok, UINT8* p_rply_ok2, UINT8* p_rply,
              UINT8  l_cmd, UINT8  l_rply_ok, UINT8  l_rply_ok2, UINT8 l_rply, void (*call_func)(void))
{
   UINT8  i;
   for (i=0; i < l_cmd; i++)
   {
      UARTTransmit(p_cmd[i]);
   }
   i = read_line();
   if ((0 == err_flag) && (i != 0) && (i != sizeof(read_buffer)))
   {
      if ((!memcmp(&read_buffer[0], p_rply_ok, l_rply_ok)) ||
          (!memcmp(&read_buffer[0], p_rply_ok2, l_rply_ok2))) //      ...
      {
         i = read_line();
         if ((0 == err_flag) && (i != 0) && (i != sizeof(read_buffer)))
         {
            if (!memcmp(&read_buffer[0], p_rply, l_rply))     //       ...
            {
               call_func();  //  
            }
         }
      }
   }
}

/***********************************
 *  -   *
 ***********************************/
UINT8 EEGetNumberOfRadios(void)
{
   return EERead(EE_NUM_OF_RADIOS);
}

/************************************
 *  -   *
 ************************************/
void EESetNumberOfRadios(UINT8 num)
{
   EEWrite(EE_NUM_OF_RADIOS, num);
}

/************************************
 *              *
 ************************************/
void EECalcAndWriteCS(void)
{
   UINT16 cs = CalcCRC16();
   EEWrite(EE_CS2, (UINT8)cs);
   EEWrite(EE_CS1, (UINT8)(cs>>8));
}

/******************************************
 *  CRC16    EEPROM *
 * X^16 + X^12 + X^5 + 1 CRC16     *
 ******************************************/
UINT16 CalcCRC16 (void)
{
   UINT32  rreg, rtemp;
   UINT16  ibyte, ibit;
   UINT16  address = EE_NUM_OF_RADIOS;
   UINT8   data_length = (EE_NUM_OF_RADIOS*3)+1;
 
   rreg = 0L;
 
   for (ibyte = 0; ibyte < data_length; ibyte++)
   {
      rtemp = ((UINT32)EERead(address++)) << (CRC_BIT_LENGTH - 7);
      for (ibit = 0; ibit < 8; ibit++)
      {
         rreg <<= 1;
         rreg ^= ( ((rtemp ^ rreg) & CRC_HIGHBIT) ? CRC_POLYN : 0);
         rtemp <<= 1;
      }
   }
 
    return ((UINT16)(rreg & CRC_MASK));
}

/************************************
 *                       *
 ************************************/
bool EECheckCS(void)
{
   bool result = false;

   if (EEGetCS() == CalcCRC16()) result = true;

   return result;
}

/************************************
 *                       *
 ************************************/
UINT16 EEGetCS(void)
{
   UINT16 cs;

   cs  = (UINT16)EERead(EE_CS2);
   cs ^= (((UINT16)EERead(EE_CS1)) << 8);

   return cs;
}

/************************************************
 *        *
 *    (  baud rate),        *
 ************************************************/
bool EEGetParam(UINT8 num, UINT8* ar, UINT16* br)
{
   UINT16 tmp_br;
   bool   result = false;
   UINT16 base_addr = (EE_INFO_START + (num*3));

   //     ""
   *ar = EERead(base_addr);
   tmp_br  = (UINT16)EERead(base_addr+1);
   tmp_br ^= (UINT16)(EERead(base_addr+2)<<8);
   *br = tmp_br;

   //  
   if (EECheckCS() && (0xFFFF != tmp_br) && (0xFF != *ar) && (0 != tmp_br) && (0 != *ar)) result = true;

   return result;
}

/***********************************************
 *        *
 *             (  baud rate)             *
 ***********************************************/
void EESetParam(UINT8 num, UINT8 ar, UINT16 br)
{
   UINT16 base_addr = (EE_INFO_START + (num*3));

   //     ""
   EEWrite(base_addr, ar);
   EEWrite(base_addr+1, (UINT8)br);
   EEWrite(base_addr+2, (UINT8)(br>>8));
}

/***********************************************
 *        *
 * (  baud rate)    , *
 *                               *
 ***********************************************/
void EEADDRadio(UINT8 curr_ar, UINT16 curr_br)
{
   UINT8  i, nr, ar;
   UINT16 br;

   for (i = NUMBER_OF_STORED_RADIOS-1; i > 0; i--)         //   9  0 
   {
      EEGetParam(i-1, &ar, &br); // i-1 -    i -
      EESetParam(i, ar, br);
   }

   EESetParam(0, curr_ar, curr_br);                         //   ""    (  0)    

   nr = EEGetNumberOfRadios();
   if (0xFF == nr) nr = 0;                                  //    EEPROM
   if (++nr > 10) nr = 10;                                  //  - ""   10 
   EESetNumberOfRadios(nr);
   EECalcAndWriteCS();
}

/*****************************
 *             *
 *****************************/
int main()
{
   UINT8  i, nor, tmp_addr, addr1;
   UINT16 tmp_br, br1;

   cli();
   IOinit();
   LCDinit();

   CLR_LEDFindConnect();
   CLR_ALL_LEDS();

   Locate(1, 0);
   lcd_print_str("   Icom CI-V    ");
   Locate(2, 0);
   lcd_print_str(" (c)UA0LTB'2012 ");
   Wait_mS(SHOW_DELAY);

   sei();

   //  
   if (EECheckCS())
   {
      nor = EEGetNumberOfRadios();

      if ((0 == nor) || (0xFF == nor)) goto start_full_search; //  - "" ,      

      //   -   ,     /
      for (i = 0; i < nor; i++)
      {
         if(EEGetParam(i, &tmp_addr, &tmp_br))
         {
            UARTinit(tmp_br);
            //  
            if(radio_check_connection(tmp_addr))
            {
               //   -   0-    ,   .   
               EEGetParam(0, &addr1, &br1);
               EESetParam(0, tmp_addr, tmp_br);
               EESetParam(i, addr1, br1);
               //  
               EECalcAndWriteCS();
               radio_address = tmp_addr;
               baud_rate     = tmp_br;
               goto run;
            }
         }
         if (0 == i) //       1-  ,  1-   ""
         {
            cls();
            lcd_print_str("      ");
            Wait_mS(SHOW_DELAY);
         }
         Locate(1, 12);
         lcd_print_str(search[i & 0x03]);
         TOGGLE_LEDFindConnect();
      }
      goto radio_not_found;
   }

start_full_search:
   //     (  : 9600, 1200, 300, 19200, 4800)
   for (i = 0; i < sizeof(baud)/sizeof(&baud[0]); i++)
   {
      if(radio_search(baud[i]))
      {
         //     "" -  
         EEADDRadio(radio_address, baud_rate);
         //    
         goto run;
      }
   }

radio_not_found:
   //   
   cls();
   Locate(1, 0);
   lcd_print_str("  ");
   Wait_mS(SHOW_DELAY);
   cls();
   goto start_full_search;

run:
   SET_LEDFindConnect(); //   ( ) -     LED_FIND_CONNECT
   cls();
   //   UART
   UARTinit(baud_rate);

   //  radio_address
   RAinit(radio_address);

   for (;;)
   {

#if (0 == _DEBUG)

      //    
      get_data(&read_freq[0],     &read_freq_rply_ok[0],     &read_freq_rply_ok2[0],     &read_freq_rply[0],
               sizeof(read_freq), sizeof(read_freq_rply_ok), sizeof(read_freq_rply_ok2), sizeof(read_freq_rply), FreqPrint);

      //     
      get_data(&read_mod[0],     &read_mod_rply_ok[0],     &read_mod_rply_ok2[0],     &read_mod_rply[0],
               sizeof(read_mod), sizeof(read_mod_rply_ok), sizeof(read_mod_rply_ok2), sizeof(read_mod_rply), ModePrint);

      //     S-
      get_data(&read_smeter[0],     &read_smeter_rply_ok[0],     &read_smeter_rply_ok2[0],     &read_smeter_rply[0],
               sizeof(read_smeter), sizeof(read_smeter_rply_ok), sizeof(read_smeter_rply_ok2), sizeof(read_smeter_rply), SMeterPrint);

      //    ""?
      if (radio_check_connection(radio_address))
      {
         SET_LEDFindConnect(); //   -    LED_FIND_CONNECT 
         if ((frequency >= 136) && (frequency <= 174)) // 144 
         {
            SET_LED_144MHZ();
            CLR_LED_430MHZ();
            CLR_LED_1200MHZ();
         }
         else if ((frequency >= 420) && (frequency <= 480)) // 430 
         {
            CLR_LED_144MHZ();
            SET_LED_430MHZ();
            CLR_LED_1200MHZ();
         }
         else if ((frequency >= 1240) && (frequency <= 1320)) // 1200 
         {
            CLR_LED_144MHZ();
            CLR_LED_430MHZ();
            SET_LED_1200MHZ();
         }
         else
         {
            CLR_ALL_LEDS();
         }
      }
      else
      {
         CLR_LEDFindConnect(); //  -   LED_FIND_CONNECT
         CLR_ALL_LEDS();       //     
      }

#else
      print_reply();

#endif //(0 == _DEBUG)

      pause();
   }
   return 0;
}
