//       C  ATmega8:
// DDRC   = 0xFF;                    //   C  
// PORTC  = 0x00;                    //    C   0

#include "common_defs.h"

const UINT16 dec_mass5[]          = {10000, 1000, 100, 10, 1};
const UINT8 PROGMEM hex_mass[]    = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

#if (1 == WINSTAR)
/************************************************
 *               *
 ************************************************/
const UINT8 xlatTable[] =
{
   -128,  02,-126,-125,-124,-123, 217, 218,-120,-119,-118,-117,-116,-115,-114,-113, // 80
   -112,-111,-110,-109,-108,-107,-106,-105,-104,-103,-102,-101,-100, -99, -98, -97, // 90
    -96, -95, -94, -93, -92, -91, -90, -89, 162,  32,  32,  32,  32,  32,  32,  32, // A0
     01,  32,  32,  32,  32,  32,  32,  32, 181,  32,  32,  32,  32,  32,  32,  32, // B0
     65, 160,  66, 161, 224,  69, 163, 164, 165, 166,  75, 167,  77,  72,  79, 168, // C0
     80,  67,  84, 169, 228,  88, 225, 171, 172, 226, 173, 174,  98, 175, 176, 177, // D0
     97, 178, 179, 180, 227, 101, 182, 183, 184, 185, 186, 187, 188, 189, 111, 190, // E0
    112,  99, 191, 121, 228, 120, 229, 192, 193, 230, 194, 195, 196, 197, 198, 199  // F0
};

/********************************
 *    *
 ********************************/
UINT8 symbolCGRAM[]=
{
//  0 -  
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */

//  1
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x04,    /*  .....O..  */
   0x00,    /*  ........  */

//  2
   0x00,    /*  ........  */
   0x10,    /*  ...O....  */
   0x08,    /*  ....O...  */
   0x04,    /*  .....O..  */
   0x02,    /*  ......O.  */
   0x01,    /*  .......O  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */

//  3
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x15,    /*  ...O.O.O  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */

//  4
   0x10,    /*  ...O....  */
   0x10,    /*  ...O....  */
   0x10,    /*  ...O....  */
   0x15,    /*  ...O.O.O  */
   0x10,    /*  ...O....  */
   0x10,    /*  ...O....  */
   0x10,    /*  ...O....  */
   0x00,    /*  ........  */

//  5
   0x14,    /*  ...O.O..  */
   0x14,    /*  ...O.O..  */
   0x14,    /*  ...O.O..  */
   0x15,    /*  ...O.O.O  */
   0x14,    /*  ...O.O..  */
   0x14,    /*  ...O.O..  */
   0x14,    /*  ...O.O..  */
   0x00,    /*  ........  */

//  6
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x15,    /*  ...O.O.O  */
   0x00,    /*  ........  */

//  7
   0x17,    /*  ...R.RRR  */
   0x14,    /*  ...R.R..  */
   0x1F,    /*  ...RRRRR  */
   0x05,    /*  .....R.R  */
   0x1D,    /*  ...RRR.R  */
   0x00,    /*  ........  */
   0x00,    /*  ........  */
   0x00     /*  ........  */
};

/**********************************
 *   CGRAM.   *
 * (  ) *
 **********************************/
void InitCGram(void)
{
   UINT8 i;

   LCD_write_cmd(0x40);                      //     CGRAM
   for (i = 0; i < sizeof(symbolCGRAM); i++)
   {
      LCD_write_data(symbolCGRAM[i]);        //     CGRAM
   }
}

/************************************************
 *     WH1604B *
 ************************************************/
UINT8 xlt(UINT8 letter)
{
   if (letter >= 0x80)
   {
      return xlatTable[letter-0x80];
   }
   else
      return letter;
}
#endif

/**********************
 *   *
 **********************/
void cls(void)
{
   LCD_write_cmd(0x01);    //         
   Wait_mS(2);             //
}

/*****************************
 *      *
 *****************************/
void SET_E(void)
{
   PORTC |= _BV(PC5);
}

/*****************************
 *          *
 *****************************/
void CLR_E(void)
{
   PORTC &= ~_BV(PC5);
}

/*****************************
 *    A0 ()   *
 *****************************/
void SET_A0(void)
{
   PORTC |= _BV(PC4);
}

/*****************************
 *       A0 ()   *
 *****************************/
void CLR_A0(void)
{
   PORTC &= ~_BV(PC4);
}

/****************************************
 *      *
 *    1,  -  0 *
 ****************************************/
void Locate(UINT8 line, UINT8 pos)
{
   UINT8 res;

   switch (line)
   {
      case 1:
         res = 0x80;
         break;
      case 2:
         res = 0xC0;
         break;
      case 3:
         res = 0x90;
         break;
      case 4:
         res = 0xD0;
         break;
      default:
         res = 0x80;
   }

   res |= pos;

   LCD_write_cmd(res);
}

/*****************************
 *      *
 * cd != 0 -           *
 * cd == 0 -          *
 *****************************/
void LCD_write(UINT8 value, UINT8 cd)
{
   if (cd)
      SET_A0();
   else
      CLR_A0();

   PORTC ^= ((value >> 4) & 0x0F);
   SET_E();
   CLR_E();
   PORTC &= 0xF0;

   PORTC ^= (value & 0x0F);
   SET_E();
   CLR_E();
   PORTC &= 0xF0;
   Wait_uS(30);
}

/*****************************
 *     *
 *****************************/
void LCD_write_data(UINT8 data)
{
#if (1 == WINSTAR)
   LCD_write(xlt(data), 1);
#else
   LCD_write(data, 1);
#endif
}

/*****************************
 *     *
 *****************************/
void LCD_write_cmd(UINT8 cmd)
{
   LCD_write(cmd, 0);
}

/*****************************
 *        *
 *****************************/
void Wait_mS(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      Wait_uS(1000);
   }
}

/*****************************
 *        *
 *****************************/
void Wait_uS(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
      __asm__ volatile("nop");
   }
}

/***********************************
 *         *
 ***********************************/
void lcd_print_str(char *text)
{
   while (*text)
   {
      LCD_write_data((UINT8)*text++);
   }
}

/*******************************************
 *      *
 *******************************************/
void lcd_print_hex_byte(UINT8 byte)
{
   LCD_write_data((UINT8)(__LPM(hex_mass + (byte >> 4))));   //   
   LCD_write_data((UINT8)(__LPM(hex_mass + (byte & 0x0F)))); //   
}

/***********************************
 *     *
 ***********************************/
void lcd_print_dec(UINT16 ciffer)
{
   UINT8         i;
   UINT8         counter;
   UINT8         print = 0;

   if (!ciffer)
   {
      lcd_print_str(" 0");
      return;
   }

   for (i = 0; i < 5; i++)
   {
      counter = 0;
      while (ciffer >= dec_mass5[i])
      {
         ciffer -= dec_mass5[i];
         counter++;
      }
      if (counter)
      {
         print = 1;
         LCD_write_data((UINT8)counter+0x30);
      }
      else if (print)
      {
         LCD_write_data((UINT8)counter+0x30);
      }
      else
      {
         lcd_print_char(' ');
      }
   }
}

/*********************
 *  LCD *
 *********************/
void LCDinit(void)
{
   CLR_E(); Wait_mS(40);
   CLR_A0(); PORTC |= 0x03; //   (8 ) (0x03 -    0x30) -       8-   
   Wait_uS(1);  //    (tAS)
   SET_E();                //     (tDSW)
   CLR_E(); Wait_uS(40);   //  
   SET_E(); Wait_uS(1);    //    E=1
   CLR_E(); Wait_uS(40);   //  
   SET_E(); Wait_uS(1);
   CLR_E(); Wait_uS(40);   //  
   PORTC &= 0xF0;          //  0x30
   PORTC |= 0x02;          //   (4 ) (0x02 -    0x20) (   RW  A0   !)
   SET_E(); Wait_uS(1);    //     (tDSW)
   CLR_E(); Wait_uS(1);    //     E=1
   PORTC &= 0xF0;
//               
   LCD_write_cmd(0x2A); Wait_uS(40);  //   
   LCD_write_cmd(0x0C);	Wait_uS(40);  // ,  
   cls();                             //        
   LCD_write_cmd(0x06); Wait_uS(40);  //   :   
#if (1 == WINSTAR)
   InitCGram();  //  WINSTAR'     -       
#endif
}
